@@ -65,9 +65,8 @@ http_auth_cache_entry_init (const struct http_auth_require_t * const require, co
      *(store pointer to http_auth_require_t, which is persistent
      * and will be different for each realm + permissions combo)*/
     http_auth_cache_entry * const ae =
-      malloc(sizeof(http_auth_cache_entry) + ulen + pwlen
-             + (k == username ? 0 : klen));
-    force_assert(ae);
+      ck_malloc(sizeof(http_auth_cache_entry) + ulen + pwlen
+                + (k == username ? 0 : klen));
     ae->require = require;
     ae->ctime = log_monotonic_secs;
     ae->dalgo = dalgo;
@@ -98,7 +97,7 @@ http_auth_cache_free (http_auth_cache *ac)
     splay_tree *sptree = ac->sptree;
     while (sptree) {
         http_auth_cache_entry_free(sptree->data);
-        sptree = splaytree_delete(sptree, sptree->key);
+        sptree = splaytree_delete_splayed_node(sptree);
     }
     free(ac);
 }
@@ -106,27 +105,27 @@ http_auth_cache_free (http_auth_cache *ac)
 static http_auth_cache *
 http_auth_cache_init (const array *opts)
 {
-    http_auth_cache *ac = malloc(sizeof(http_auth_cache));
-    force_assert(ac);
+    http_auth_cache *ac = ck_malloc(sizeof(http_auth_cache));
     ac->sptree = NULL;
     ac->max_age = 600; /* 10 mins */
     for (uint32_t i = 0, used = opts->used; i < used; ++i) {
         data_unset *du = opts->data[i];
         if (buffer_is_equal_string(&du->key, CONST_STR_LEN("max-age")))
             ac->max_age = (time_t)
-              config_plugin_value_to_int32(du, (int32_t)ac->max_age);
+              config_plugin_value_to_int32(du, 600); /* 10 min if invalid num */
     }
     return ac;
 }
 
+__attribute_pure__
 static int
 http_auth_cache_hash (const struct http_auth_require_t * const require, const char *username, const uint32_t ulen)
 {
+    /* (similar to splaytree_djbhash(), but with two strings hashed) */
     uint32_t h = /*(hash pointer value, which includes realm and permissions)*/
       djbhash((char *)(intptr_t)require, sizeof(intptr_t), DJBHASH_INIT);
     h = djbhash(username, ulen, h);
-    /* strip highest bit of hash value for splaytree (see splaytree_djbhash())*/
-    return (int32_t)(h & ~(((uint32_t)1) << 31));
+    return (int32_t)h;
 }
 
 static http_auth_cache_entry *
@@ -143,7 +142,7 @@ http_auth_cache_insert (splay_tree ** const sptree, const int ndx, void * const
      * and splaytree has not been modified since http_auth_cache_query())*/
     /* *sptree = splaytree_splay(*sptree, ndx); */
     if (NULL == *sptree || (*sptree)->key != ndx)
-        *sptree = splaytree_insert(*sptree, ndx, data);
+        *sptree = splaytree_insert_splayed(*sptree, ndx, data);
     else { /* collision; replace old entry */
         data_free_fn((*sptree)->data);
         (*sptree)->data = data;
@@ -178,12 +177,9 @@ mod_auth_periodic_cleanup(splay_tree **sptree_ptr, const time_t max_age, const u
         max_ndx = 0;
         mod_auth_tag_old_entries(sptree, keys, &max_ndx, max_age, cur_ts);
         for (i = 0; i < max_ndx; ++i) {
-            int ndx = keys[i];
-            sptree = splaytree_splay(sptree, ndx);
-            if (sptree && sptree->key == ndx) {
-                http_auth_cache_entry_free(sptree->data);
-                sptree = splaytree_delete(sptree, ndx);
-            }
+            sptree = splaytree_splay_nonnull(sptree, keys[i]);
+            http_auth_cache_entry_free(sptree->data);
+            sptree = splaytree_delete_splayed_node(sptree);
         }
     } while (max_ndx == sizeof(keys)/sizeof(int));
     *sptree_ptr = sptree;
@@ -224,8 +220,7 @@ INIT_FUNC(mod_auth_init) {
 	static http_auth_scheme_t http_auth_scheme_basic  = { "basic",  mod_auth_check_basic,  NULL };
 	static http_auth_scheme_t http_auth_scheme_digest = { "digest", mod_auth_check_digest, NULL };
 	static const http_auth_scheme_t http_auth_scheme_extern = { "extern", mod_auth_check_extern, NULL };
-	plugin_data *p = calloc(1, sizeof(*p));
-	force_assert(p);
+	plugin_data *p = ck_calloc(1, sizeof(*p));
 
 	/* register http_auth_scheme_* */
 	http_auth_scheme_basic.p_d = p;
@@ -276,6 +271,7 @@ static void data_auth_free(data_unset *d)
     free(dauth);
 }
 
+__attribute_returns_nonnull__
 static data_auth *data_auth_init(void)
 {
     static const struct data_methods fn = {
@@ -283,8 +279,7 @@ static data_auth *data_auth_init(void)
       data_auth_free,
       NULL, /* insert_dup must not be called on this data */
     };
-    data_auth * const dauth = calloc(1, sizeof(*dauth));
-    force_assert(NULL != dauth);
+    data_auth * const dauth = ck_calloc(1, sizeof(*dauth));
     dauth->type       = TYPE_OTHER;
     dauth->fn         = &fn;
 
@@ -535,6 +530,18 @@ static handler_t mod_auth_require_parse_array(const array *value, array * const
 				return HANDLER_ERROR;
 			}
 
+			for (uint32_t o = 0; o < n; ++o) {
+				const buffer *k = &((data_array *)value->data[o])->key;
+				if (buffer_clen(&da_file->key) >= buffer_clen(k)
+				    && 0 == strncmp(da_file->key.ptr, k->ptr, buffer_clen(k))) {
+					log_error(errh, __FILE__, __LINE__,
+					  "auth.require path (\"%s\") will never match due to "
+					  "earlier match (\"%s\"); fix by sorting longer paths "
+					  "before shorter paths", da_file->key.ptr, k->ptr);
+					break;
+				}
+			}
+
 			if (require) { /*(always true at this point)*/
 				data_auth * const dauth = data_auth_init();
 				buffer_copy_buffer(&dauth->key, &da_file->key);
@@ -695,6 +702,8 @@ static handler_t mod_auth_uri_handler(request_st * const r, void *p_d) {
 }
 
 
+__attribute_cold__
+__declspec_dllexport__
 int mod_auth_plugin_init(plugin *p);
 int mod_auth_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
@@ -852,7 +861,7 @@ mod_auth_check_basic(request_st * const r, void *p_d, const struct http_auth_req
     default:
         log_error(r->conf.errh, __FILE__, __LINE__,
           "password doesn't match for %s username: %s IP: %s",
-          r->uri.path.ptr, user, r->con->dst_addr_buf.ptr);
+          r->uri.path.ptr, user, r->dst_addr_buf->ptr);
         r->keep_alive = -1; /*(disable keep-alive if bad password)*/
         rc = mod_auth_send_401_unauthorized_basic(r, require->realm);
         break;
@@ -1423,7 +1432,7 @@ mod_auth_digest_validate_params (request_st * const r, const struct http_auth_re
         log_error(r->conf.errh, __FILE__, __LINE__,
           "digest: auth failed: uri mismatch (%s != %.*s), IP: %s",
           r->target_orig.ptr, (int)dp->len[e_uri], dp->ptr[e_uri],
-          r->con->dst_addr_buf.ptr);
+          r->dst_addr_buf->ptr);
         return mod_auth_send_400_bad_request(r);
     }
 
@@ -1536,7 +1545,7 @@ mod_auth_check_digest (request_st * const r, void *p_d, const struct http_auth_r
         /* digest not ok */
         log_error(r->conf.errh, __FILE__, __LINE__,
           "digest: auth failed for %.*s: wrong password, IP: %s",
-          (int)ai.ulen, ai.username, r->con->dst_addr_buf.ptr);
+          (int)ai.ulen, ai.username, r->dst_addr_buf->ptr);
         r->keep_alive = -1; /*(disable keep-alive if bad password)*/
         return mod_auth_send_401_unauthorized_digest(r, require, 0);
     }
